from ase.io import read,write
import sys
#sys.path.append('/path/to/kaldo')
from kaldo.conductivity import Conductivity
from kaldo.forceconstants import ForceConstants
from kaldo.observables.harmonic_with_q import HarmonicWithQ
from kaldo.phonons import Phonons
import numpy as np
from kaldo.controllers import plotter
import pickle
import os
from sklearn.neighbors import KernelDensity
import matplotlib.pyplot as plt



N=4
cond_list=[]
dr_list=[1,2]#glass sample to treat
for T in [300]: #temperatures
    for dr in dr_list:
    	for eta in [0.02,0.03,0.04,0.05,0.06,0.07,0.08,0.09,0.1,0.2,0.3,0.4,0.5,0.6]: #Allen-Feldman smearing
            # Replicate the unit cell 'nrep'=1 time
            nrep = 1
            supercell = np.array([nrep, nrep, nrep])
    
            # Load in computed 2nd, 3rd IFCs from LAMMPS outputs
            forceconstants = ForceConstants.from_folder(folder='N{}/{}/'.format(N,dr),supercell=supercell,format='numpy',only_second=True)
            #forceconstants.second.save('second',format='numpy')   
    
    
            # Configure phonon object
            # 'is_classic': specify if the system is classic, True for classical and False for quantum
            # 'temperature: temperature (Kelvin) at which simulation is performed
            # 'folder': name of folder containing phonon property and thermal conductivity calculations
            # 'storage': Format to storage phonon properties ('formatted' for ASCII format data, 'numpy' 
            #            for python numpy array and 'memory' for quick calculations, no data stored)
    
            phonons_config = {'is_classic': False, 
                              'temperature': T, #'temperature'=300K
                              'folder': 'ald_{}_{}/'.format(N,dr),
                               'third_bandwidth':0.5/4.135, # 0.5 eV is used here.
                               'broadening_shape':'triangle',
                               'storage': 'numpy'}
    
            # Set up phonon object by passing in configuration details and the forceconstants object computed above
            phonons = Phonons(forceconstants=forceconstants, **phonons_config)
            freq=phonons.frequency
            qhgk_cond = Conductivity(phonons=phonons, method='qhgk', storage='numpy')
            qhgk_cond.diffusivity_bandwidth = round(eta,5)
            print('Conductivity from QHGK (W/m-K): %.3f' % (np.mean(np.diag(qhgk_cond.conductivity.sum(axis=0)))))
            print(qhgk_cond.conductivity.sum(axis=0))
            cond_list.append( np.mean(np.diag(qhgk_cond.conductivity.sum(axis=0))) )
print(cond_list)

